@@ -80,7 +80,6 @@ from airflow.providers.cncf.kubernetes.utils.pod_manager import (
     PodNotFoundException,
     PodOperatorHookProtocol,
     PodPhase,
-    check_exception_is_kubernetes_api_unauthorized,
     container_is_succeeded,
     get_container_termination_message,
 )
@@ -113,6 +112,10 @@ class PodReattachFailure(AirflowException):
     """When we expect to be able to find a pod but cannot."""
 
 
+class PodCredentialsExpiredFailure(AirflowException):
+    """When pod fails to refresh credentials."""
+
+
 class KubernetesPodOperator(BaseOperator):
     """
     Execute a task in a Kubernetes Pod.
@@ -495,6 +498,7 @@ class KubernetesPodOperator(BaseOperator):
         if include_try_number:
             labels.update(try_number=ti.try_number)
         # In the case of sub dags this is just useful
+        # TODO: Remove this when the minimum version of Airflow is bumped to 3.0
         if getattr(context["dag"], "parent_dag", False):
             labels["parent_dag_id"] = context["dag"].parent_dag.dag_id  # type: ignore[attr-defined]
 
@@ -651,9 +655,8 @@ class KubernetesPodOperator(BaseOperator):
             return result
 
     @tenacity.retry(
-        stop=tenacity.stop_after_attempt(3),
         wait=tenacity.wait_exponential(max=15),
-        retry=tenacity.retry_if_exception(lambda exc: check_exception_is_kubernetes_api_unauthorized(exc)),
+        retry=tenacity.retry_if_exception_type(PodCredentialsExpiredFailure),
         reraise=True,
     )
     def await_pod_completion(self, pod: k8s.V1Pod):
@@ -674,6 +677,10 @@ class KubernetesPodOperator(BaseOperator):
                     "Failed to check container status due to permission error. Refreshing credentials and retrying."
                 )
                 self._refresh_cached_properties()
+                self.pod_manager.read_pod(
+                    pod=pod
+                )  # attempt using refreshed credentials, raises if still invalid
+                raise PodCredentialsExpiredFailure("Kubernetes credentials expired, retrying after refresh.")
             raise exc
 
     def _refresh_cached_properties(self):
@@ -783,9 +790,7 @@ class KubernetesPodOperator(BaseOperator):
                         since_time=last_log_time,
                     )
 
-                    if pod_log_status.running:
-                        self.log.info("Container still running; deferring again.")
-                        self.invoke_defer_method(pod_log_status.last_log_time)
+                    self.invoke_defer_method(pod_log_status.last_log_time)
                 else:
                     self.invoke_defer_method()
 
