@@ -45,7 +45,6 @@ from airflow.exceptions import (
 )
 from airflow.providers.cncf.kubernetes.kubernetes_helper_functions import (
     POD_NAME_MAX_LENGTH,
-    add_pod_suffix,
     add_unique_suffix,
     rand_str,
 )
@@ -156,12 +155,15 @@ class PodGenerator:
         # Attach sidecar
         self.extract_xcom = extract_xcom
 
-    @deprecated(reason="This function is deprecated.", category=AirflowProviderDeprecationWarning)
+    @deprecated(
+        reason="This method is deprecated and will be removed in the future releases",
+        category=AirflowProviderDeprecationWarning,
+    )
     def gen_pod(self) -> k8s.V1Pod:
         """Generate pod."""
         result = self.ud_pod
 
-        result.metadata.name = add_pod_suffix(pod_name=result.metadata.name)
+        result.metadata.name = add_unique_suffix(name=result.metadata.name)
 
         if self.extract_xcom:
             result = self.add_xcom_sidecar(result)
@@ -210,8 +212,8 @@ class PodGenerator:
             return k8s_object
         elif isinstance(k8s_legacy_object, dict):
             warnings.warn(
-                "Using a dictionary for the executor_config is deprecated and will soon be removed."
-                'please use a `kubernetes.client.models.V1Pod` class with a "pod_override" key'
+                "Using a dictionary for the executor_config is deprecated and will soon be removed. "
+                'Please use a `kubernetes.client.models.V1Pod` class with a "pod_override" key'
                 " instead. ",
                 category=AirflowProviderDeprecationWarning,
                 stacklevel=2,
@@ -475,6 +477,7 @@ class PodGenerator:
         execution_date=None,
         run_id=None,
         airflow_worker=None,
+        include_version=False,
     ):
         """
         Generate selector for kubernetes executor pod.
@@ -489,6 +492,7 @@ class PodGenerator:
             execution_date=execution_date,
             run_id=run_id,
             airflow_worker=airflow_worker,
+            include_version=include_version,
         )
         label_strings = [f"{label_id}={label}" for label_id, label in sorted(labels.items())]
         selector = ",".join(label_strings)
@@ -507,6 +511,7 @@ class PodGenerator:
         map_index=None,
         execution_date=None,
         run_id=None,
+        include_version=True,
     ):
         """
         Generate labels for kubernetes executor pod.
@@ -518,8 +523,9 @@ class PodGenerator:
             "task_id": make_safe_label_value(task_id),
             "try_number": str(try_number),
             "kubernetes_executor": "True",
-            "airflow_version": airflow_version.replace("+", "-"),
         }
+        if include_version:
+            labels["airflow_version"] = airflow_version.replace("+", "-")
         if airflow_worker is not None:
             labels["airflow-worker"] = make_safe_label_value(str(airflow_worker))
         if map_index is not None and map_index >= 0:
@@ -575,7 +581,7 @@ class PodGenerator:
 
     @staticmethod
     @deprecated(
-        reason="This function is deprecated. Use `add_pod_suffix` in `kubernetes_helper_functions`.",
+        reason="This method is deprecated. Use `add_pod_suffix` in `kubernetes_helper_functions`.",
         category=AirflowProviderDeprecationWarning,
     )
     def make_unique_pod_id(pod_id: str) -> str | None:
@@ -595,12 +601,6 @@ class PodGenerator:
         :param pod_id: requested pod name
         :return: ``str`` valid Pod name of appropriate length
         """
-        warnings.warn(
-            "This function is deprecated. Use `add_pod_suffix` in `kubernetes_helper_functions`.",
-            AirflowProviderDeprecationWarning,
-            stacklevel=2,
-        )
-
         if not pod_id:
             return None
 
